package com.zhongshu.reward.server.core.service;

import com.github.microservice.auth.security.helper.AuthHelper;
import com.github.microservice.core.util.result.content.ResultContent;
import com.github.microservice.core.util.result.content.ResultState;
import com.zhongshu.reward.client.model.VipUserModel;
import com.zhongshu.reward.client.model.wallet.WalletReceiptsModel;
import com.zhongshu.reward.client.type.ReceiptsStatus;
import com.zhongshu.reward.client.type.ReceiptsType;
import com.zhongshu.reward.client.type.UserType;
import com.zhongshu.reward.server.core.dao.InviteRecordDao;
import com.zhongshu.reward.server.core.dao.VipUserRecordDao;
import com.zhongshu.reward.server.core.dao.WalletDao;
import com.zhongshu.reward.server.core.dao.WalletReceiptsDao;
import com.zhongshu.reward.server.core.domain.InviteRecord;
import com.zhongshu.reward.server.core.domain.VipUserRecord;
import com.zhongshu.reward.server.core.domain.Wallet;
import com.zhongshu.reward.server.core.domain.WalletReceipts;
import com.zhongshu.reward.server.core.util.DateUtils;
import com.zhongshu.vip.client.model.param.VipUserParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author wjf
 * @date 2024/8/6
 */
@Service
@Slf4j
public class WalletReceiptsService {

    @Autowired
    WalletDao walletDao;

    @Autowired
    WalletService walletService;

    @Autowired
    WalletReceiptsDao walletReceiptsDao;

    @Autowired
    VipUserRecordDao vipUserRecordDao;

    @Autowired
    InviteRecordDao inviteRecordDao;

    @Autowired
    AuthHelper authHelper;


    /**
     * 邀请返利入账
     * @param
     * @return
     */
    @Transactional
    public void receipts(VipUserParam vipUserParam){

        VipUserRecord vipUserRecord = new VipUserRecord();
        BeanUtils.copyProperties(vipUserParam, vipUserRecord, "createTime");
        vipUserRecord.setOperateTime(DateUtils.timeToLong(vipUserParam.getCreateTime(), DateUtils.FORMAT_LONG));
        VipUserRecord save = vipUserRecordDao.save(vipUserRecord);
        //查询邀请关系
        InviteRecord inviteRecord = inviteRecordDao.findTopByUserIdOrderByCreateTime(vipUserParam.getCpId());
        if (Objects.isNull(inviteRecord)){
            return;
        }
        if(inviteRecord.getUserType().equals(UserType.OLD)){//是否老用户
            return;
        }

        //todo 校验对应套餐返利规则
        BigDecimal total = BigDecimal.ZERO;

        if (vipUserRecord.getSuc().equals(1) || vipUserRecord.getSuc().equals(4)){
            //订阅入账
            WalletReceipts walletReceipts = new WalletReceipts();
            //获取邀请人userid
            String inviteId = inviteRecord.getInviteUserId();
            Wallet wallet = walletService.getWalletByUserId(inviteId);
            walletReceipts.setWallet(wallet);
            walletReceipts.setUserId(vipUserParam.getCpId());
            walletReceipts.setReceiptsType(ReceiptsType.COMMISSION);
            walletReceipts.setOutTradeNo(vipUserParam.getPlanningId());
            walletReceipts.setTotal(total);
            walletReceipts.setEstimatedTime(DateUtils.nextMonthDayStartTime(25));
            walletReceipts.setVipUserRecord(save);
            //邀请人钱包
            if (walletReceipts.getReceiptsType()==null){
                walletReceipts.setStatus(ReceiptsStatus.WAIT);
            }
            wallet.setWaitAmount(wallet.getWaitAmount().add(total));
            walletDao.save(wallet);
            walletReceiptsDao.save(walletReceipts);
        }
    }

    /**
     * 取消返利入账
     * @param
     * @return
     */
    @Transactional
    public Object cancelReceipts(String outTradeNo){

        WalletReceipts walletReceipts = walletReceiptsDao.findTopByOutTradeNo(outTradeNo);
        if (ObjectUtils.isEmpty(walletReceipts)){
            return ResultContent.build(ResultState.Fail,"返利订单不存在或未入账");
        }
        if (!walletReceipts.getStatus().equals(ReceiptsStatus.WAIT)){
            return ResultContent.build(ResultState.Fail,"订单不处于未结算状态");
        }

        walletReceipts.setStatus(ReceiptsStatus.CANCEL);
        walletReceiptsDao.save(walletReceipts);
        Wallet wallet = walletDao.findTop1ById(walletReceipts.getWallet().getId());
        wallet.setWaitAmount(wallet.getWaitAmount().subtract(walletReceipts.getTotal()));
        walletDao.save(wallet);
        return ResultContent.build(ResultState.Success);
    }

    public Object listByWallet(){
        String userId = authHelper.getCurrentUser().getUserId();
        Wallet wallet = walletService.getWalletByUserId(userId);
        List<WalletReceipts> list = walletReceiptsDao.findByWallet_IdOrderByCreateTimeDesc(wallet.getId());
        return ResultContent.buildContent(list.stream().map(this::toModel).collect(Collectors.toList()));
    }

    public Object receiptsDetail(String receiptsId){
        WalletReceipts walletReceipts = walletReceiptsDao.findTop1ById(receiptsId);
        if (ObjectUtils.isEmpty(walletReceipts)){
            return ResultContent.build(ResultState.Fail, "记录不存在");
        }
        VipUserRecord vipUserRecord = walletReceipts.getVipUserRecord();
        VipUserModel vipUserModel = new VipUserModel();
        BeanUtils.copyProperties(vipUserRecord, vipUserModel);
        BeanUtils.copyProperties(walletReceipts, vipUserModel);
        return ResultContent.buildContent(vipUserModel);
    }

    public Object receiptsDetail(Integer year, Integer month){
        String userId = authHelper.getCurrentUser().getUserId();
        Long monthStartTime = DateUtils.getMonthStartTime(year, month);
        Long monthEndTime = DateUtils.getMonthEndTime(year, month);
        List<WalletReceipts> walletReceipts = walletReceiptsDao.inviteDetail(userId, monthStartTime, monthEndTime);
        return ResultContent.buildContent(walletReceipts.stream().map(this::toVipUserModel).collect(Collectors.toList()));
    }

    private VipUserModel toVipUserModel(WalletReceipts walletReceipts) {
        VipUserModel vipUserModel = new VipUserModel();
        if (ObjectUtils.isNotEmpty(walletReceipts)){
            VipUserRecord vipUserRecord = walletReceipts.getVipUserRecord();
            BeanUtils.copyProperties(vipUserRecord, vipUserModel);
            BeanUtils.copyProperties(walletReceipts, vipUserModel);
        }
        return vipUserModel;
    }

    WalletReceiptsModel toModel(WalletReceipts walletReceipts){
        WalletReceiptsModel model = new WalletReceiptsModel();
        if (null != walletReceipts){
            BeanUtils.copyProperties(walletReceipts, model, "wallet");
            model.setWalletId(walletReceipts.getWallet().getId());
        }
        return model;
    }
}
